@@ -2,7 +2,7 @@
  *
  * Information about ssh channels.
  *
- * $Id: channel.h,v 1.60 2000/10/31 21:35:36 nisse Exp $
+ * $Id: channel.h,v 1.64 2001/02/13 17:41:57 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -34,6 +34,31 @@
 #include "server_pty.h"
 #include "write_buffer.h"
 
+struct channel_open_info
+{
+  UINT32 type_length;
+
+  /* NOTE: This is a pointer into the packet, so if it is needed later
+   * it must be copied. */
+  const UINT8 *type_data;
+  
+  int type;
+
+  UINT32 remote_channel_number;
+  UINT32 send_window_size;
+  UINT32 send_max_packet;
+};
+
+struct channel_request_info
+{
+  UINT32 type_length;
+  const UINT8 *type_data;
+  
+  int type;
+
+  int want_reply;
+};
+
 #define GABA_DECLARE
 #include "channel.h.x"
 #undef GABA_DECLARE
@@ -53,7 +78,7 @@
 
 /* Means that we should send close when we have both sent and received EOF. */
 #define CHANNEL_CLOSE_AT_EOF 0x10
-
+  
 
 /* GABA:
    (class
@@ -99,6 +124,9 @@
   
        (request_types object alist)
 
+       ; If non-NULL, invoked for unknown channel requests.
+       (request_fallback object channel_request)
+       
        (flags . int)
 
        ; Number of files connected to this channel. For instance,
@@ -123,8 +151,6 @@
        (eof method void)
   
        ; Reply from SSH_MSG_CHANNEL_OPEN_REQUEST
-       ;; (open_confirm method int)
-       ;; (open_failure method int)
        (open_continuation object command_continuation)
 
        ; Queue of channel requests that we expect replies on
@@ -134,9 +160,7 @@
        ; in the right order
        (active_requests struct object_queue)))
        
-       ; Reply from SSH_MSG_CHANNEL_REQUEST 
-       ;; (channel_success method int)
-       ;; (channel_failure method int))) */
+*/
 
 #define CHANNEL_RECEIVE(s, t, d) \
 ((s)->receive((s), (t), (d)))
@@ -171,6 +195,11 @@
        (global_requests object alist)
        ; Channel types that we can open
        (channel_types object alist)
+
+       ; FIXME: Add fallbacks, to be used for unknown requests
+       ; and channel types.
+       ; (global_fallback object ...)
+       (open_fallback object channel_open)
        
        ; Allocation of local channel numbers is managed using the same
        ; method as is traditionally used for allocation of unix file 
@@ -180,11 +209,20 @@
        ; assigned to them. So the channels table is not enough for keeping track of which
        ; numbers are in use.
        (in_use space UINT8)
-       
+
+       ; Allocated size of the arrays.
        (allocated_channels . UINT32)
-       (next_channel . UINT32)
-     
+
+       ; Number of entries in the arrays that are in use and
+       ; initialized.
        (used_channels . UINT32)
+
+       ; The smallest channel number that is likely to be free
+       (next_channel . UINT32)
+
+       ; Number of currently allocated channel numbers.
+       (channel_count . UINT32)
+       
        (max_channels . UINT32) ; Max number of channels allowed 
 
        ; Forwarded TCP ports
@@ -221,7 +259,7 @@
 #define GLOBAL_REQUEST(r, c, t, w, a, n, e) ((r)->handler((r), (c), (t), (w), (a), (n), (e)))
 
 /* SSH_MSG_CHANNEL_OPEN */
-
+  
 /* Raised if opening of a channel fails. Used both on the client and
  * the server side.*/
 /* GABA:
@@ -242,18 +280,17 @@ make_channel_open_exception(UINT32 error_code, const char *msg);
      (vars
        (handler method void
                 "struct ssh_connection *connection"
-		"UINT32 type"
-		"UINT32 send_window_size"
-		"UINT32 send_max_packet"
+		"struct channel_open_info *info"
                 "struct simple_buffer *data"
                 "struct command_continuation *c"
 		"struct exception_handler *e")))
 */
 
-#define CHANNEL_OPEN(o, c, t, w, m, d, r, e) \
-((o)->handler((o), (c), (t), (w), (m), (d), (r), (e)))
+#define CHANNEL_OPEN(o, c, i, d, r, e) \
+((o)->handler((o), (c), (i), (d), (r), (e)))
 
 /* SSH_MSG_CHANNEL_REQUEST */
+
 /* GABA:
    (class
      (name channel_request)
@@ -261,15 +298,14 @@ make_channel_open_exception(UINT32 error_code, const char *msg);
        (handler method void
 		"struct ssh_channel *channel"
 		"struct ssh_connection *connection"
-		"UINT32 type"
-		"int want_reply"
+		"struct channel_request_info *info"
 		"struct simple_buffer *args"
 		"struct command_continuation *c"
 		"struct exception_handler *e")))
 */
 
-#define CHANNEL_REQUEST(s, c, conn, t, w, a, n, e) \
-((s)->handler((s), (c), (conn), (t), (w), (a), (n), (e)))
+#define CHANNEL_REQUEST(s, c, conn, i, a, n, e) \
+((s)->handler((s), (c), (conn), (i), (a), (n), (e)))
 
 /* #define CONNECTION_START(c, s) ((c)->start((c), (s))) */
 
@@ -319,15 +355,35 @@ void
 channel_start_receive(struct ssh_channel *channel,
 		      UINT32 initial_window_size);
 
+struct lsh_string *
+format_channel_open_s(struct lsh_string *type,
+		      UINT32 local_channel_number,
+		      struct ssh_channel *channel,
+		      struct lsh_string *args);
+
+#if 0
+struct lsh_string *
+format_channel_open_a(int type,
+		      UINT32 local_channel_number,
+		      struct ssh_channel *channel,
+		      struct lsh_string *args);
+#endif
+
 struct lsh_string *
 format_channel_open(int type, UINT32 local_channel_number,
 		    struct ssh_channel *channel,
 		    const char *format, ...);
 
-struct lsh_string *format_channel_request(int type,
-					  struct ssh_channel *channel,
-					  int want_reply,
-					  const char *format, ...);
+struct lsh_string *
+format_channel_request_i(struct channel_request_info *info,
+			 struct ssh_channel *channel,
+			 UINT32 args_length, const UINT8 *args_data);
+
+struct lsh_string *
+format_channel_request(int type,
+		       struct ssh_channel *channel,
+		       int want_reply,
+		       const char *format, ...);
 
 struct lsh_string *
 format_global_request(int type, int want_reply,
