@@ -1,6 +1,6 @@
 /* client.c
  *
- * $Id: client.c,v 1.104 2001/01/30 23:06:24 nisse Exp $ */
+ * $Id: client.c,v 1.105 2001/02/18 22:22:07 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -924,12 +924,6 @@ client_argp_parser(int key, char *arg, struct argp_state *state)
       if (options->start_shell)
 	client_add_action(options, client_shell_session(options));
 
-      if (object_queue_is_empty(&options->actions))
-	{
-	  argp_error(state, "No actions given.");
-	  break;
-	}
-
       break;
 
     case 'p':
