@@ -1,6 +1,6 @@
 /* io.c
  *
- * $Id: io.c,v 1.121 2001/01/31 16:19:38 nisse Exp $ */
+ * $Id: io.c,v 1.125 2001/02/22 10:12:21 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -74,7 +74,7 @@
      (super resource)
      (vars
        (next object lsh_signal_handler)
-       (flag . "sig_atomic_t *")
+       (flag . "volatile sig_atomic_t *")
        (action object lsh_callback)))
 */
 
@@ -438,7 +438,7 @@ make_io_backend(void)
 
 struct resource *
 io_signal_handler(struct io_backend *b,
-		  sig_atomic_t *flag,
+		  volatile sig_atomic_t *flag,
 		  struct lsh_callback *action)
 {
   NEW(lsh_signal_handler, handler);
@@ -560,7 +560,9 @@ do_buffered_read(struct io_callback *s,
       assert(fd->want_read);
       assert(self->handler);
 
-      close_fd_nicely(fd);
+      /* Close the fd, unless it has a write callback. */
+      close_fd_read(fd);
+      
       READ_HANDLER(self->handler, 0, NULL);
     }
 	
@@ -586,16 +588,18 @@ do_consuming_read(struct io_callback *c,
   CAST_SUBTYPE(io_consuming_read, self, c);
   UINT32 wanted = READ_QUERY(self);
 
+  assert(fd->want_read);
+
+  if (fd->hanged_up)
+    {
+      /* If hanged_up is set, pretend that read() returned 0 */
+      goto eof;
+    }
+  
   if (!wanted)
     {
       fd->want_read = 0;
     }
-  else if (fd->hanged_up)
-    {
-      /* If hanged_up is set, pretend that read() returned 0 */
-      close_fd_nicely(fd);
-      A_WRITE(self->consumer, NULL);
-    }
   else
     {
       struct lsh_string *s = lsh_string_alloc(wanted);
@@ -627,10 +631,11 @@ do_consuming_read(struct io_callback *c,
 	}
       else
 	{
-	  close_fd_nicely(fd);
+	eof:
+	  /* Close the fd, unless it has a write callback. */
+	  close_fd_read(fd);
 	  A_WRITE(self->consumer, NULL);
 	}
-      
     }
 }
 
@@ -868,7 +873,8 @@ static void do_kill_fd(struct resource *r)
 {
   CAST_SUBTYPE(lsh_fd, fd, r);
 
-  /* FIXME: Should we use close_fd() or close_fd_nicely() ? */
+  /* We use close_fd_nicely, so that any data in the write buffer is
+   * flushed before the fd is closed. */
   if (r->alive)
     close_fd_nicely(fd);
 }
@@ -1127,6 +1133,10 @@ sockaddr2info(size_t addr_len UNUSED,
 	return info;
       }
 #endif
+    case AF_UNIX:
+      /* Silently return NULL. This happens when a gateway client
+       * connects. */
+      return NULL;
     default:
       werror("io.c: sockaddr2info(): Unsupported address family.\n");
       return NULL;
@@ -1758,6 +1768,16 @@ void close_fd_nicely(struct lsh_fd *fd)
     kill_fd(fd);
 }
 
+/* Stop reading, but if the fd has a write callback, keep it open. */
+void close_fd_read(struct lsh_fd *fd)
+{
+  fd->want_read = 0;
+  fd->read = NULL;
+  
+  if (!fd->write)
+    /* We won't be writing anything on this fd, so close it. */
+    kill_fd(fd);
+}
 
 /* Responsible for handling the EXC_FINISH_READ exception. It should
  * be a parent to the connection related exception handlers, as for
