@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: io.h,v 1.71 2001/01/30 23:08:12 nisse Exp $ */
+ * $Id: io.h,v 1.73 2001/02/22 10:12:21 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -228,7 +228,7 @@ void io_run(struct io_backend *b);
 
 struct resource *
 io_signal_handler(struct io_backend *b,
-		  sig_atomic_t *flag,
+		  volatile sig_atomic_t *flag,
 		  struct lsh_callback *action);
 
 struct resource *
@@ -340,6 +340,9 @@ void close_fd(struct lsh_fd *fd);
  * is completely written. */
 void close_fd_nicely(struct lsh_fd *fd);
 
+/* Stop reading, but if the fd has a write callback, keep it open. */
+void close_fd_read(struct lsh_fd *fd);
+
 struct lsh_fd *io_write_file(struct io_backend *backend,
 			    const char *fname, int flags,
 			    int mode,
