@@ -2,7 +2,7 @@
  *
  * client main program
  *
- * $Id: lsh.c,v 1.131 2001/01/24 21:15:27 nisse Exp $ */
+ * $Id: lsh.c,v 1.133 2001/02/20 23:12:53 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -266,11 +266,13 @@ do_options2identities(struct command *ignored UNUSED,
       COMMAND_RETURN(c, make_object_list(0, -1));
     }
   else
-    COMMAND_CALL(make_spki_read_userkeys(options->algorithms->algorithms,
-					 options->signature_algorithms,
-					 options->super.tty),
-		 f, c, e);
-  
+    {
+      struct command *command
+	= make_spki_read_userkeys(options->algorithms->algorithms,
+				  options->signature_algorithms,
+				  options->super.tty);
+      COMMAND_CALL(command, f, c, e);
+    }
   lsh_string_free(tmp);
 }
 
@@ -832,6 +834,9 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	lsh_string_free(tmp);
       }
 
+      /* We can't add the gateway action immediately when the -G
+       * option is encountered, as we need the name and port of the
+       * remote machine (self->super.remote) first. */
       if (self->start_gateway)
 	{
 	  struct local_info *gateway;
@@ -857,6 +862,12 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 			    (make_listen_local(self->super.backend, gateway)));
 	}
 
+      if (object_queue_is_empty(&self->super.actions))
+	{
+	  argp_error(state, "No actions given.");
+	  break;
+	}
+      
       /* Start background poll */
       RANDOM_POLL_BACKGROUND(self->random->poller);
       
