@@ -2,7 +2,7 @@
  *
  * Interact with the user.
  *
- * $Id: unix_interact.c,v 1.2 2000/11/20 20:24:21 nisse Exp $ */
+ * $Id: unix_interact.c,v 1.3 2001/02/20 17:49:20 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -48,7 +48,7 @@
 
 #include "unix_interact.c.x"
 
-static sig_atomic_t window_changed;
+static volatile sig_atomic_t window_changed;
 
 static void winch_handler(int signum)
 {
